@@ -1,11 +1,10 @@
 from __future__ import annotations
 from functools import wraps as functools_wraps
-from typing import Any, Callable, TYPE_CHECKING, Iterable, TypeVar, overload
+from typing import Any, Callable, TYPE_CHECKING, Iterable, TypeVar, overload, MutableSequence
 from typing_extensions import _AnnotatedAlias
 import inspect
 from enum import Enum
 import warnings
-from collections.abc import MutableSequence
 from docstring_parser import parse, compose
 
 from magicgui.events import Signal
@@ -58,6 +57,20 @@ _RESERVED = {"__magicclass_parent__", "__magicclass_children__", "_close_on_run"
              }
 
 def check_override(cls: type):
+    """
+    Some of the methods should not be overriden because they are essential for Magic class
+    construction.
+
+    Parameters
+    ----------
+    cls : type
+        Base class to test override.
+
+    Raises
+    ------
+    AttributeError
+        If forbidden override found.
+    """    
     subclass_members = set(cls.__dict__.keys())
     collision = subclass_members & _RESERVED
     if collision:
@@ -226,13 +239,14 @@ class MagicTemplate:
                 func = method._function
             else:
                 func = method
+                
             if template is not None:
-                func = wraps(template)(func)
-            if hasattr(cls, method.__name__):
-                getattr(cls, method.__name__).__signature__ = get_signature(func)
+                wraps(template)(func)
+            if hasattr(cls, func.__name__):
+                getattr(cls, func.__name__).__signature__ = get_signature(func)
             
             upgrade_signature(func, additional_options={"into": cls.__name__})
-            return func
+            return method
         
         return wrapper if method is None else wrapper(method)
     
@@ -264,7 +278,7 @@ class MagicTemplate:
                     index = _get_index(child_instance, name)
                     new = False
                 except ValueError:
-                    index = len(child_instance)
+                    index = -1
                     new = True
                 
                 self.append(widget)
@@ -275,18 +289,23 @@ class MagicTemplate:
                     else:
                         del child_instance[index]
                         child_instance.insert(index, widget)
+                    
                 else:
                     widget.visible = False
                     if new:
-                        widget = child_instance._create_widget_from_method(lambda x: None)
-                        child_instance.append(widget)
+                        child_widget = child_instance._create_widget_from_method(lambda x: None)
+                        child_widget.text = widget.text
+                        child_instance.append(child_widget)
                     else:
                         child_widget: PushButtonPlus | AbstractAction = child_instance[index]
-                        child_widget.changed.disconnect()
-                        child_widget.changed.connect(widget.changed)
-                        child_widget.tooltip = widget.tooltip
-                        child_widget._doc = widget._doc
+                    
+                    child_widget.changed.disconnect()
+                    child_widget.changed.connect(widget.changed)
+                    child_widget.tooltip = widget.tooltip
+                    child_widget._doc = widget._doc
+                    
                 widget._unwrapped = True
+                
                 break
         else:
             raise RuntimeError(f"{child_clsname} not found in class {self.__class__.__name__}")
@@ -427,8 +446,6 @@ class MagicTemplate:
             def run_function():
                 mgui = _build_mgui(widget, func)
                 if mgui.call_count == 0 and len(mgui.called._slots) == 0:
-                    mgui.native.setParent(self.native, mgui.native.windowFlags())
-                    mgui.native.move(screen_center() - mgui.native.rect().center())
                     
                     # deal with popup mode.
                     if self._popup_mode not in (PopUpMode.popup, PopUpMode.dock):
@@ -437,7 +454,7 @@ class MagicTemplate:
                         mgui.margins = (0, 0, 0, 0)
                         title = Separator(orientation="horizontal", text=text, button=True)
                         title.btn_text = "-"
-                        title.btn_clicked.connect(mgui.hide)
+                        title.btn_clicked.connect(mgui.hide) # TODO: should remove mgui from self?
                         mgui.insert(0, title)
                         mgui.append(Separator(orientation="horizontal"))
                         
@@ -445,17 +462,19 @@ class MagicTemplate:
                             parent_self = self._search_parent_magicclass()
                             parent_self.append(mgui)
                         elif self._popup_mode == PopUpMode.first:
-                            self.insert(0, mgui)
+                            child_self = _child_that_has_widget(self, obj, widget)
+                            child_self.insert(0, mgui)
                         elif self._popup_mode == PopUpMode.last:
-                            self.append(mgui)
+                            child_self = _child_that_has_widget(self, obj, widget)
+                            child_self.append(mgui)
                         elif self._popup_mode == PopUpMode.above:
-                            name = _get_widget_name(widget)
-                            i = _get_index(self, name)
-                            self.insert(i, mgui)
+                            child_self = _child_that_has_widget(self, obj, widget)
+                            i = _get_index(child_self, widget)
+                            child_self.insert(i, mgui)
                         elif self._popup_mode == PopUpMode.below:
-                            name = _get_widget_name(widget)
-                            i = _get_index(self, name)
-                            self.insert(i+1, mgui)
+                            child_self = _child_that_has_widget(self, obj, widget)
+                            i = _get_index(child_self, widget)
+                            child_self.insert(i+1, mgui)
                             
                     elif self._popup_mode == PopUpMode.dock:
                         parent_self = self._search_parent_magicclass()
@@ -480,7 +499,12 @@ class MagicTemplate:
                             dock = viewer.window.add_dock_widget(
                                 mgui, name=_get_widget_name(widget), area="right"
                                 )
-                    
+                    else:
+                        # To be popped up correctly, window flags of FunctionGui should be
+                        # "windowFlags" and should appear at the center.
+                        mgui.native.setParent(self.native, mgui.native.windowFlags())
+                        mgui.native.move(screen_center() - mgui.native.rect().center())
+                        
                     if self._close_on_run:
                         if self._popup_mode != PopUpMode.dock:
                             mgui.called.connect(mgui.hide)
@@ -703,7 +727,7 @@ def _n_parameters(func: Callable):
     """    
     return len(inspect.signature(func).parameters)
 
-def _get_index(container: Container, widget_or_name: Widget | str):
+def _get_index(container: Container, widget_or_name: Widget | str) -> int:
     """
     Identical to container[widget_or_name], which sometimes doesn't work
     in magic-class.
@@ -719,6 +743,19 @@ def _get_index(container: Container, widget_or_name: Widget | str):
         raise ValueError(f"{widget_or_name} not found in {container}")
     return i
 
+def _child_that_has_widget(self: BaseGui, method: Callable, 
+                           widget_or_name: Widget | str) -> BaseGui:
+    child_clsname = get_additional_option(method, "into")
+    if child_clsname is None:
+        return self
+    for child_instance in self._iter_child_magicclasses():
+        if child_instance.__class__.__name__ == child_clsname:
+            break
+    else:
+        raise ValueError(f"{widget_or_name} not found.")
+    return child_instance
+            
+
 def _method_as_getter(self, bound_value: Callable):
     *clsnames, funcname = bound_value.__qualname__.split(".")
     
