@@ -20,6 +20,19 @@ from ..fields import MagicField
 from ..widgets import Separator, FreeWidget
 from ..utils import iter_members
 
+def _check_popupmode(popup_mode: PopUpMode):
+    if popup_mode in (PopUpMode.above, PopUpMode.below, PopUpMode.first):
+        msg = f"magicmenu does not support popup mode {popup_mode.value}."\
+                "PopUpMode.popup is used instead"
+        warnings.warn(msg, UserWarning)
+    elif popup_mode == PopUpMode.last:
+        msg = f"magicmenu does not support popup mode {popup_mode.value}."\
+                "PopUpMode.parentlast is used instead"
+        warnings.warn(msg, UserWarning)
+        popup_mode = PopUpMode.parentlast
+    
+    return popup_mode
+
 class MenuGuiBase(BaseGui, MutableSequence):
     _component_class = Action
     changed = Signal(object)
@@ -32,15 +45,7 @@ class MenuGuiBase(BaseGui, MutableSequence):
                  error_mode: str|ErrorMode = None,
                  labels: bool = True
                  ):
-        if popup_mode in (PopUpMode.above, PopUpMode.below, PopUpMode.first):
-            msg = f"magicmenu does not support popup mode {popup_mode.value}."\
-                   "PopUpMode.popup is used instead"
-            warnings.warn(msg, UserWarning)
-        elif popup_mode == PopUpMode.last:
-            msg = f"magicmenu does not support popup mode {popup_mode.value}."\
-                   "PopUpMode.parentlast is used instead"
-            warnings.warn(msg, UserWarning)
-            popup_mode = PopUpMode.parentlast
+        popup_mode = _check_popupmode(popup_mode)
             
         super().__init__(close_on_run=close_on_run, popup_mode=popup_mode, error_mode=error_mode)
         name = name or self.__class__.__name__
@@ -241,6 +246,44 @@ class MenuGuiBase(BaseGui, MutableSequence):
                 if labeled_widget:
                     labeled_widget.label_width = widest_label
 
+    def render(self):
+        try:
+            import numpy as np
+        except ImportError:
+            raise ModuleNotFoundError(
+                "could not find module 'numpy'. "
+                "Please `pip install numpy` to render widgets."
+            ) from None
+        import qtpy
+        img = self.native.grab().toImage()
+        bits = img.constBits()
+        h, w, c = img.height(), img.width(), 4
+        if qtpy.API_NAME == "PySide2":
+            arr = np.array(bits).reshape(h, w, c)
+        else:
+            bits.setsize(h * w * c)
+            arr = np.frombuffer(bits, np.uint8).reshape(h, w, c)
+
+        return arr[:, :, [2, 1, 0, 3]]
+
+    def _repr_png_(self):
+        """Return PNG representation of the widget for QtConsole."""
+        from io import BytesIO
+
+        try:
+            from imageio import imsave
+        except ImportError:
+            print(
+                "(For a nicer magicmenu widget representation in "
+                "Jupyter, please `pip install imageio`)"
+            )
+            return None
+
+        with BytesIO() as file_obj:
+            imsave(file_obj, self.render(), format="png")
+            file_obj.seek(0)
+            return file_obj.read()
+
 def _insert(qmenu: QMenu, key: int, obj):
     """
     Insert a QObject into a QMenu in a Pythonic way, like qmenu.insert(key, obj).
